from CvPythonExtensions import *
from WriterUtils import *

import CvScreenEnums
import CvUtil
import CvConfigParser
import CvPath
import os
import os.path
import Popup as PyPopup
import shutil 

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

# Change this to change the location where the resulting CIV4ProcessInfos.xml file 
# will be saved. So if the default value is left to "Test" then the 
# CIV4ProcessInfos.xml file would be written in a directory called "Test" inside 
# of the Tech Tree Editor mod folder.
# Default value is Test
g_strSaveDirectoryLocation = "Test"

# Change the value to false if the assets directory structure shouldn't be 
# created at the directory specified by the "Save Directory Location".
# Default value is True
g_bCreateAssetHierarchy = True

# Change the value to false if the changes made shouldn't be saved into the 
# assets directory for the tech editor. Changing it to false will require 
# manual copying of the saved files from the directory specified in the 
# "Save Directory Location" configurable option to the editor's asset directory 
# when wanting to edit existing mod files.
# Default value is True
g_bSaveChangesInEditorAssetsDirectory = True

def loadConfigurationValues():
	global g_strSaveDirectoryLocation
	global g_bCreateAssetHierarchy
	global g_bSaveChangesInEditorAssetsDirectory
	
	config = CvConfigParser.CvConfigParser("Tech Tree Editor Config.ini")

	if(config != None):
		g_strSaveDirectoryLocation = config.get("Tech Tree Editor", "Save Directory Location", "Test")
		g_bCreateAssetHierarchy = config.getboolean("Tech Tree Editor", "Create Asset Hierarchy", True)
		g_bSaveChangesInEditorAssetsDirectory = config.getboolean("Tech Tree Editor", "Save Changes In Editor Assets Directory", True)

		if(len(g_strSaveDirectoryLocation) == 0):
			g_strSaveDirectoryLocation = "Test"
			
class ProcessInfoWriter:
		
	def __init__(self):
		loadConfigurationValues()
	
				
	def isProductionToCommerceModifiersDefined(self, iProcess):
		objProcessInfo = gc.getProcessInfo(iProcess)
		
		for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
			if(objProcessInfo.getProductionToCommerceModifier(iI) > 0):
				return true

		return false
		
	def printProcess(self, iProcess):
		objProcessInfo = gc.getProcessInfo(iProcess)

		szText = ""
		
		szText = appendString(szText,"		<ProcessInfo>")
		szText = appendString(szText,"			<Type>%s</Type>" %(objProcessInfo.getType()))
		szText = appendString(szText,"			<Description>%s</Description>" %(objProcessInfo.getTextKey()))
		szText = appendString(szText,"			<Strategy>%s</Strategy>" %(objProcessInfo.getStrategyKey()))

		if(objProcessInfo.getTechPrereq() != -1):
			szText = appendString(szText,"			<TechPrereq>%s</TechPrereq>" %(gc.getTechInfo(objProcessInfo.getTechPrereq()).getType()))
		else:
			szText = appendString(szText,"			<TechPrereq>NONE</TechPrereq>")

		if(self.isProductionToCommerceModifiersDefined(iProcess)):
			szText = appendString(szText,"			<ProductionToCommerceModifiers>")
			for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
				szText = appendString(szText,"				<iProductionToCommerceModifier>%s</iProductionToCommerceModifier>" %(objProcessInfo.getProductionToCommerceModifier(iI)))
			szText = appendString(szText,"			</ProductionToCommerceModifiers>")
		else:
			szText = appendString(szText,"			<ProductionToCommerceModifiers/>")

		szText = appendString(szText,"			<Button>%s</Button>" %(objProcessInfo.getButton()))

		szText = appendString(szText,"		</ProcessInfo>")

		
			
		return szText

						
	def printProcesss(self):	
	
		strSavePath = ""
		strFileSavePath = ""
		strModPath = ""
		strModAssetPath = ""
		strModAssetSavePath = ""
		strModAssetPath = ""
		
		if(os.access(CvPath.userActiveModDir,os.F_OK)):
			strModPath = CvPath.userActiveModDir
		elif(os.access(CvPath.installActiveModDir,os.F_OK)):
			strModPath = CvPath.installActiveModDir
		else:
			self.dataSavePopup(false)
			return
			
		strSavePath = "%s\%s" %(strModPath,g_strSaveDirectoryLocation)
		strModAssetPath = "%s\\Assets\\" %(strModPath)
		
		if(g_bCreateAssetHierarchy):
			strSavePath += "\Assets\XML\Units" 

		strFileSavePath = "%s\%s" %(strSavePath, "CIV4ProcessInfo.xml")

		try:
			if(os.access(strSavePath,os.F_OK)):
				if(os.access(strFileSavePath,os.F_OK)):
					os.remove(strFileSavePath)
				os.removedirs(strSavePath)
		except:
			pass
			
		if(not os.access(strSavePath,os.F_OK)):
			os.makedirs(strSavePath)

							
		szText = "<?xml version=\"1.0\"?>"
		szText = appendString(szText,"<!-- Created by Tech Civilizeditor %s by TheLopez-->" %(editorVersion()))
		szText = appendString(szText,"<!-- Sid Meier's Civilization 4 -->")
		szText = appendString(szText,"<!-- Copyright Firaxis Games 2005 -->")
		szText = appendString(szText,"<!-- -->")
		szText = appendString(szText,"<!-- Process -->")
		szText = appendString(szText,"<Civ4ProcessInfo xmlns=\"x-schema:CIV4GameInfoSchema.xml\">")
		szText = appendString(szText,"	<ProcessInfos>")
				
		for iProcess in range(gc.getNumProcessInfos()):

			if(gc.getProcessInfo(iProcess).isDeleted()):
				continue

			szText = appendString(szText,self.printProcess(iProcess))
			
						
		szText = appendString(szText,"	</ProcessInfos>")
		szText = appendString(szText,"</Civ4ProcessInfo>")

			
		fd = os.open(strFileSavePath, os.O_CREAT)
		os.close(fd)

		fd = os.open(strFileSavePath, os.O_RDWR)
		os.write(fd, szText)
		os.close(fd)
		
		if(g_bSaveChangesInEditorAssetsDirectory):
			strModAssetPath = "%s\%s" %(strModPath,"\Assets\XML\GameInfo")
			strModAssetSavePath = "%s\%s" %(strModPath, "\Assets\XML\GameInfo\CIV4ProcessInfo.xml")
			try:
				if(os.access(strModAssetPath,os.F_OK)):
					if(os.access(strModAssetSavePath,os.F_OK)):
						os.remove(strModAssetSavePath)
			except:
				pass
				
			if(not os.access(strModAssetPath,os.F_OK)):
				os.makedirs(strModAssetPath)

			fd = os.open(strModAssetSavePath, os.O_CREAT)
			os.close(fd)

			fd = os.open(strModAssetSavePath, os.O_RDWR)
			os.write(fd, szText)
			os.close(fd)
	

