from CvPythonExtensions import *
from WriterUtils import *

import CvScreenEnums
import CvUtil
import CvConfigParser
import CvPath
import os
import os.path
import Popup as PyPopup
import shutil 

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

# Change this to change the location where the resulting CIV4ReligionInfos.xml file 
# will be saved. So if the default value is left to "Test" then the 
# CIV4ReligionInfos.xml file would be written in a directory called "Test" inside 
# of the Tech Tree Editor mod folder.
# Default value is Test
g_strSaveDirectoryLocation = "Test"

# Change the value to false if the assets directory structure shouldn't be 
# created at the directory specified by the "Save Directory Location".
# Default value is True
g_bCreateAssetHierarchy = True

# Change the value to false if the changes made shouldn't be saved into the 
# assets directory for the tech editor. Changing it to false will require 
# manual copying of the saved files from the directory specified in the 
# "Save Directory Location" configurable option to the editor's asset directory 
# when wanting to edit existing mod files.
# Default value is True
g_bSaveChangesInEditorAssetsDirectory = True

def loadConfigurationValues():
	global g_strSaveDirectoryLocation
	global g_bCreateAssetHierarchy
	global g_bSaveChangesInEditorAssetsDirectory
	
	config = CvConfigParser.CvConfigParser("Tech Tree Editor Config.ini")

	if(config != None):
		g_strSaveDirectoryLocation = config.get("Tech Tree Editor", "Save Directory Location", "Test")
		g_bCreateAssetHierarchy = config.getboolean("Tech Tree Editor", "Create Asset Hierarchy", True)
		g_bSaveChangesInEditorAssetsDirectory = config.getboolean("Tech Tree Editor", "Save Changes In Editor Assets Directory", True)

		if(len(g_strSaveDirectoryLocation) == 0):
			g_strSaveDirectoryLocation = "Test"
			
class ReligionInfoWriter:
		
	def __init__(self):
		loadConfigurationValues()
		
	def isGlobalReligionCommercesDefined(self, iReligion):
		objReligionInfo = gc.getReligionInfo(iReligion)
		
		for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
			if(objReligionInfo.getGlobalReligionCommerce(iI)):
				return true
		
	def isHolyCityCommercesDefined(self, iReligion):
		objReligionInfo = gc.getReligionInfo(iReligion)
		
		for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
			if(objReligionInfo.getHolyCityCommerce(iI)):
				return true
		
	def isStateReligionCommercesDefined(self, iReligion):
		objReligionInfo = gc.getReligionInfo(iReligion)
		
		for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
			if(objReligionInfo.getStateReligionCommerce(iI)):
				return true

		return false
		
	def printReligion(self, iReligion):
		objReligionInfo = gc.getReligionInfo(iReligion)

		szText = ""
		
		szText = appendString(szText,"		<ReligionInfo>")
		szText = appendString(szText,"			<Type>%s</Type>" %(objReligionInfo.getType()))
		szText = appendString(szText,"			<Description>%s</Description>" %(objReligionInfo.getTextKey()))
		szText = appendString(szText,"			<Civilopedia>%s</Civilopedia>" %(objReligionInfo.getCivilopediaKey()))

		if(objReligionInfo.getTechPrereq() != -1):
			szText = appendString(szText,"			<TechPrereq>%s</TechPrereq>" %(gc.getTechInfo(objReligionInfo.getTechPrereq()).getType()))
		else:
			szText = appendString(szText,"			<TechPrereq>NONE</TechPrereq>")

		if(objReligionInfo.getFreeUnitClass() != -1):
			szText = appendString(szText,"			<FreeUnitClass>%s</FreeUnitClass>" %(gc.getUnitClassInfo(objReligionInfo.getFreeUnitClass()).getType()))
		else:
			szText = appendString(szText,"			<FreeUnitClass>NONE</FreeUnitClass>")

		szText = appendString(szText,"			<iSpreadFactor>%s</iSpreadFactor>" %(objReligionInfo.getSpreadFactor()))

		if(self.isGlobalReligionCommercesDefined(iReligion)):
			szText = appendString(szText,"			<GlobalReligionCommerces>")
			for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
				szText = appendString(szText,"				<iGlobalReligionCommerce>%s</iGlobalReligionCommerce>" %(objReligionInfo.getGlobalReligionCommerce(iI)))

			szText = appendString(szText,"			</GlobalReligionCommerces>")
		else:
			szText = appendString(szText,"			<GlobalReligionCommerces/>")
		
		if(self.isHolyCityCommercesDefined(iReligion)):
			szText = appendString(szText,"			<HolyCityCommerces>")
			for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
				szText = appendString(szText,"				<iHolyCityCommerce>%s</iHolyCityCommerce>" %(objReligionInfo.getHolyCityCommerce(iI)))

			szText = appendString(szText,"			</HolyCityCommerces>")
		else:
			szText = appendString(szText,"			<HolyCityCommerces/>")
		
		if(self.isStateReligionCommercesDefined(iReligion)):
			szText = appendString(szText,"			<StateReligionCommerces>")
			for iI in range(CommerceTypes.NUM_COMMERCE_TYPES):
				szText = appendString(szText,"				<iStateReligionCommerce>%s</iStateReligionCommerce>" %(objReligionInfo.getStateReligionCommerce(iI)))

			szText = appendString(szText,"			</StateReligionCommerces>")
		else:
			szText = appendString(szText,"			<StateReligionCommerces/>")
				
		szText = appendString(szText,"			<Button>%s</Button>" %(objReligionInfo.getButton()))
		szText = appendString(szText,"			<TechButton>%s</TechButton>" %(objReligionInfo.getTechButton()))
		szText = appendString(szText,"			<MovieFile>%s</MovieFile>" %(objReligionInfo.getMovieFile()))
		szText = appendString(szText,"			<MovieSound>%s</MovieSound>" %(objReligionInfo.getMovieSound()))
		szText = appendString(szText,"			<Sound>%s</Sound>" %(objReligionInfo.getSound()))
		
		szText = appendString(szText,"		</ReligionInfo>")

		
			
		return szText

						
	def printReligions(self):	
	
		strSavePath = ""
		strFileSavePath = ""
		strModPath = ""
		strModAssetPath = ""
		strModAssetSavePath = ""
		strModAssetPath = ""
		
		if(os.access(CvPath.userActiveModDir,os.F_OK)):
			strModPath = CvPath.userActiveModDir
		elif(os.access(CvPath.installActiveModDir,os.F_OK)):
			strModPath = CvPath.installActiveModDir
		else:
			self.dataSavePopup(false)
			return
			
		strSavePath = "%s\%s" %(strModPath,g_strSaveDirectoryLocation)
		strModAssetPath = "%s\\Assets\\" %(strModPath)
		
		if(g_bCreateAssetHierarchy):
			strSavePath += "\Assets\XML\Units" 

		strFileSavePath = "%s\%s" %(strSavePath, "CIV4ReligionInfo.xml")

		try:
			if(os.access(strSavePath,os.F_OK)):
				if(os.access(strFileSavePath,os.F_OK)):
					os.remove(strFileSavePath)
				os.removedirs(strSavePath)
		except:
			pass
			
		if(not os.access(strSavePath,os.F_OK)):
			os.makedirs(strSavePath)

							
		szText = "<?xml version=\"1.0\"?>"
		szText = appendString(szText,"<!-- Created by Tech Civilizeditor %s by TheLopez-->" %(editorVersion()))
		szText = appendString(szText,"<!-- Sid Meier's Civilization 4 -->")
		szText = appendString(szText,"<!-- Copyright Firaxis Games 2005 -->")
		szText = appendString(szText,"<!-- -->")
		szText = appendString(szText,"<!-- Religion -->")

		szText = appendString(szText,"<Civ4ReligionInfo xmlns=\"x-schema:CIV4GameInfoSchema.xml\">")
		szText = appendString(szText,"	<ReligionInfos>")
				
		for iReligion in range(gc.getNumReligionInfos()):

			if(gc.getReligionInfo(iReligion).isDeleted()):
				continue

			szText = appendString(szText,self.printReligion(iReligion))
			
						
		szText = appendString(szText,"	</ReligionInfos>")
		szText = appendString(szText,"</Civ4ReligionInfo>")

			
		fd = os.open(strFileSavePath, os.O_CREAT)
		os.close(fd)

		fd = os.open(strFileSavePath, os.O_RDWR)
		os.write(fd, szText)
		os.close(fd)
		
		if(g_bSaveChangesInEditorAssetsDirectory):
			strModAssetPath = "%s\%s" %(strModPath,"\Assets\XML\GameInfo")
			strModAssetSavePath = "%s\%s" %(strModPath, "\Assets\XML\GameInfo\CIV4ReligionInfo.xml")
			try:
				if(os.access(strModAssetPath,os.F_OK)):
					if(os.access(strModAssetSavePath,os.F_OK)):
						os.remove(strModAssetSavePath)
			except:
				pass
				
			if(not os.access(strModAssetPath,os.F_OK)):
				os.makedirs(strModAssetPath)

			fd = os.open(strModAssetSavePath, os.O_CREAT)
			os.close(fd)

			fd = os.open(strModAssetSavePath, os.O_RDWR)
			os.write(fd, szText)
			os.close(fd)
	

